//=============================================================================

#include <iostream>
#include <string>
#include <vector>
#include "hanoi.hpp"
using namespace std;

//-----------------------------------------------------------------------------

Hanoi::Hanoi(int cs) : countSlices{ cs }
{
   for (int i = countSlices; i > 0; --i)
   {
      tower1.push_back(i);
   }
}

//-----------------------------------------------------------------------------

bool Hanoi::isIndexAllowed(int idx, const tower_t& tower)
{
   return idx < tower.size();
}

//-----------------------------------------------------------------------------

void Hanoi::printLevelOfTower(int level, const tower_t& tower)
{
   int idx = countSlices - level;
   int slice = isIndexAllowed(idx, tower) ? tower[idx] : 0;

   string spaces(countSlices - slice, ' ');
   string halfslice(slice, '-');
   cout << spaces << halfslice << '|' << halfslice << spaces;
}

//-----------------------------------------------------------------------------

void Hanoi::printLevelOfAllTowers(int level)
{
   cout << ' ';
   printLevelOfTower(level, tower1);
   cout << ' ';
   printLevelOfTower(level, tower2);
   cout << ' ';
   printLevelOfTower(level, tower3);
   cout << '\n';
}

//-----------------------------------------------------------------------------

void Hanoi::printBasePlate()
{
   int width = 7 + 6 * countSlices;
   string plate(width, '*');
   cout << plate << '\n';
}

//-----------------------------------------------------------------------------

void Hanoi::printGame()
{
   cout << '\n';
   cout << ++counter << ".\n\n";

   for (int level = 1; level <= countSlices; ++level)
   {
      printLevelOfAllTowers(level);
   }
   printBasePlate();
}

//-----------------------------------------------------------------------------

void Hanoi::moveSlice(tower_t& src, tower_t& dest)
{
   int slice = src.back();
   src.pop_back();
   dest.push_back(slice);

   printGame();
}

//-----------------------------------------------------------------------------

void Hanoi::moveTower(int size, tower_t& src, tower_t& dest, tower_t& util)
{
   if (size == 1)
   {
      moveSlice(src, dest);
      return;
   }

   moveTower(size - 1, src, util, dest);
   moveSlice(src, dest);
   moveTower(size - 1, util, dest, src);
}

//-----------------------------------------------------------------------------

void Hanoi::run()
{
   moveTower(tower1.size(), tower1, tower3, tower2);
}
