#include "tictactoe.hpp"
using namespace std;

TicTacToe::TicTacToe()
{
   mFields.push_back({ State::empty, State::empty, State::empty });
   mFields.push_back({ State::empty, State::empty, State::empty });
   mFields.push_back({ State::empty, State::empty, State::empty });
}

ostream& operator<<(ostream& out, const TicTacToe& ttt)
{
   for (const auto& row : ttt.mFields)
   {
      for (auto s : row)
      {
         switch (s)
         {
         case TicTacToe::State::empty:
            out << " _ ";
            break;
         case TicTacToe::State::white:
            out << " O ";
            break;
         case TicTacToe::State::black:
            out << " X ";
            break;
         }
      }
      out << '\n';
   }
   return out << '\n';
}

bool TicTacToe::isNotFree(int row, int col) const
{
   return mFields[row][col] != State::empty;
}

void TicTacToe::set(const Move& m)
{
   const int row = m.getRow();
   const int col = m.getCol();
   const GameColor gc = m.getGameColor();
   const State s = convert(gc);
   mFields[row][col] = s;
}

TicTacToe::State TicTacToe::convert(GameColor gc)
{
   return gc == GameColor::white ? State::white : State::black;
}

bool TicTacToe::hasWin(GameColor gc) const
{
   const State s = convert(gc);

   if (mFields[0][0] == s && mFields[0][1] == s && mFields[0][2] == s) return true;
   if (mFields[1][0] == s && mFields[1][1] == s && mFields[1][2] == s) return true;
   if (mFields[2][0] == s && mFields[2][1] == s && mFields[2][2] == s) return true;

   if (mFields[0][0] == s && mFields[1][0] == s && mFields[2][0] == s) return true;
   if (mFields[0][1] == s && mFields[1][1] == s && mFields[2][1] == s) return true;
   if (mFields[0][2] == s && mFields[1][2] == s && mFields[2][2] == s) return true;

   if (mFields[0][0] == s && mFields[1][1] == s && mFields[2][2] == s) return true;
   if (mFields[0][2] == s && mFields[1][1] == s && mFields[2][0] == s) return true;

   return false;
}

bool TicTacToe::isFull() const
{
   for (const auto& row : mFields)
   {
      for (auto s : row)
      {
         if (s == State::empty) return false;
      }
   }
   return true;
}
