//=============================================================================

#include <iostream>
#include <map>
#include <string>
using namespace std;

//-----------------------------------------------------------------------------

using db_t = map<string, string>;

//-----------------------------------------------------------------------------

auto menu = R"xxx(Bitte waehlen Sie eine Aktion:
- e : Programmende
- l : Alle Eintraege auflisten
- s : Nach Eintrag suchen
- n : Neuen Eintrag eingeben
> )xxx";

//-----------------------------------------------------------------------------

db_t createDB()
{
   db_t db;
   db.insert({ "Lea", "1123" });
   db.insert({ "Max", "0257 / 88956" });
   db.insert({ "Eva", "888" });
   db.insert({ "Edgar", "030 / 8976" });
   db.insert({ "Edmund", "4455 / 88 99" });
   db.insert({ "Erwin", "0171 24 56 889" });
   db.insert({ "Tom", "987" });
   db.insert({ "Det", "+49 030 / 45 56 89 56" });
   db.insert({ "Eberhard", "4 5 4879897" });
   return db;
}

//-----------------------------------------------------------------------------

void printDB(const db_t& db)
{
   cout << db.size() << " Kontakte vorhanden:\n";
   for (auto& [name, no] : db)
   {
      cout << "- " << name << " => " << no << '\n';
   }
}

//-----------------------------------------------------------------------------

void searchInDB(const db_t& db)
{
   string pattern;
   cout << "Wonach soll gesucht werden: ";
   getline(cin, pattern);
   if (pattern.empty())
   {
      cout << "Suchvorgang abgebrochen\n";
      return;
   }

   auto fit = db.lower_bound(pattern);
   if (fit == cend(db))
   {
      cout << pattern << " ist nicht in der Datenbank vorhanden\n";
      return;
   }
   for (; fit != cend(db); ++fit)
   {
      if (!fit->first.starts_with(pattern)) break;
      cout << "- " << fit->first << " => " << fit->second << '\n';
   }
}

//-----------------------------------------------------------------------------

void newEntryInDB(db_t& db)
{
   cout << "Neuer Eintrag soll hinzugefuegt werden\n";

   string name;
   cout << "- Name: ";
   getline(cin, name);
   if (name.empty())
   {
      cout << "=> Abbruch\n";
      return;
   }

   string no;
   cout << "- Nummer: ";
   getline(cin, no);
   if (no.empty())
   {
      cout << "=> Abbruch\n";
      return;
   }

   auto res = db.insert({ name, no });
   if (res.second)
   {
      cout << "Datenbank enthaelt nun " << db.size() << " Kontakte\n";
   }
   else
   {
      cout << "Kontakt war schon vorhanden - kein neuer Eintrag eingefuegt\n";
   }
}

//-----------------------------------------------------------------------------

int main()
{
   cout << "WS 2024/25 - Telefonbuch 2\n";

   db_t db = createDB();
   for (string input;;)
   {
      cout << "\n" << menu;
      getline(cin, input);
      cout << '\n';

      if (input.empty()) continue;
      const char c = input[0];
      switch (c)
      {
      case 'e':
         cout << "Programmende...\n\n";
         return 0;
      case 'l':
         printDB(db);
         break;
      case 's':
         searchInDB(db);
         break;
      case 'n':
         newEntryInDB(db);
         break;
      }
   }

}
